/*******************************************************************************
 *
 *                          Messenger Android Frontend
 *                        (C) 2013-2016 Nikolai Kudashov
 *                           (C) 2017 Björn Petersen
 *                    Contact: r10s@b44t.com, http://b44t.com
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see http://www.gnu.org/licenses/ .
 *
 ******************************************************************************/


package com.b44t.messenger;

import java.util.ArrayList;
import java.util.HashMap;

public class TLRPC {

    public static final int MESSAGE_FLAG_FWD                = 0x00000004;
    public static final int MESSAGE_FLAG_REPLY              = 0x00000008;
    public static final int MESSAGE_FLAG_HAS_FROM_ID        = 0x00000100;
    public static final int MESSAGE_FLAG_HAS_MEDIA          = 0x00000200;

	public static class DraftMessage extends TLObject {
		public int flags;
		public int reply_to_msg_id;
		public String message;
		public int date;
	}

    public static class ChatPhoto extends TLObject {
		public FileLocation photo_small;
		public FileLocation photo_big;
	}

	public static class DocumentAttribute extends TLObject {
		public int w;
		public int h;
		public int duration;
		public String alt;
		public InputStickerSet stickerset;
		public int flags;
		public boolean voice;
		public String title;
		public String performer;
		public byte[] waveform;
		public String file_name;
	}

	public static class TL_documentAttributeAnimated extends DocumentAttribute {
	}

	public static class TL_documentAttributeImageSize extends DocumentAttribute {
	}

	public static class TL_documentAttributeSticker extends DocumentAttribute {
	}

	public static class TL_documentAttributeVideo extends DocumentAttribute {
	}

	public static class TL_documentAttributeAudio extends DocumentAttribute {
	}

	public static class TL_documentAttributeFilename extends DocumentAttribute {
	}

    public static class messages_Messages extends TLObject {
        public ArrayList<Message> messages = new ArrayList<>();
        public ArrayList<Chat> chats = new ArrayList<>();
        public ArrayList<User> users = new ArrayList<>();
        public int flags;
        public int pts;
        public int count;
    }

    public static class TL_messages_messages extends messages_Messages {
    }

	public static class Peer extends TLObject {
		public final int channel_id = 0;
		public int user_id;
		public final int chat_id = 0;
	}

	public static class TL_peerUser extends Peer {
	}

	public static class GeoPoint extends TLObject {
		public double _long;
		public double lat;
	}

	public static class TL_geoPoint extends GeoPoint {
	}

	public static class TL_messageMediaEmpty extends MessageMedia {
	}

	public static class TL_messageMediaVenue extends MessageMedia {
	}

	public static class TL_messageMediaDocument extends MessageMedia {
	}

	public static class TL_messageMediaContact extends MessageMedia {
	}

	public static class TL_messageMediaPhoto extends MessageMedia {
	}

	public static class TL_messageMediaGeo extends MessageMedia {
	}

	public static class PeerNotifySettings extends TLObject {
		public int flags;
		public boolean silent;
		public int mute_until;
		public String sound;
	}

	public static class TL_peerNotifySettings extends PeerNotifySettings {
	}

	public static class TL_messages_stickerSet extends TLObject {
		public StickerSet set;
		public ArrayList<Document> documents = new ArrayList<>();
	}

	public static class InputGeoPoint extends TLObject {
		public double lat;
		public double _long;
	}

	public static class TL_inputGeoPoint extends InputGeoPoint {
	}

	public static class Audio extends TLObject {
		public long id;
		public int date;
		public int duration;
		public String mime_type;
		public int size;
		public int dc_id;
		public int user_id;
		public byte[] key;
		public byte[] iv;
	}

	public static class ChatFull extends TLObject {
		public int flags;
		public int id;
		public String about;
	}

	public static class WebPage extends TLObject {
		public int flags;
		public long id;
		public String url;
		public String type;
		public String title;
		public String description;
		public Photo photo;
		public int duration;
		public String author;
		public int date;
		public Document document;
	}

    public static class InputFileLocation extends TLObject {
		public long id;
		public long access_hash;
		public long volume_id;
		public int local_id;
		public long secret;
	}

	public static class TL_inputDocumentFileLocation extends InputFileLocation {
	}

	public static class TL_inputFileLocation extends InputFileLocation {
	}

	public static class User extends TLObject {
		public int id;
		public String first_name;
		public String last_name;
		public String username;
        public String phone;
		public UserProfilePhoto photo;
        public int flags;
		public boolean self;
		public boolean contact;
		public boolean min;
    }

	public static class TL_userContact_old2 extends User {
	}

	public static class Video extends TLObject {
		public long id;
		public int user_id;
		public int date;
		public int duration;
		public int size;
		public PhotoSize thumb;
		public int dc_id;
		public int w;
		public int h;
		public String mime_type;
		public String caption;
		public byte[] key;
		public byte[] iv;
	}

	public static class InputDocument extends TLObject {
		public long id;
		public long access_hash;
	}

	public static class TL_inputDocument extends InputDocument {
	}

	public static class Document extends TLObject {
		public long id;
		public long access_hash;
		public int user_id;
		public int date;
		public String file_name;
		public String mime_type;
		public int size;
		public PhotoSize thumb;
		public int dc_id;
		public byte[] key;
		public byte[] iv;
		public String caption;
		public ArrayList<DocumentAttribute> attributes = new ArrayList<>();
		public String mr_path;
	}

	public static class TL_document extends Document {
	}

	public static class InputMedia extends TLObject {
		public InputFile file;
		public InputFile thumb;
		public String mime_type;
		public ArrayList<DocumentAttribute> attributes = new ArrayList<>();
		public String caption;
		public InputGeoPoint geo_point;
		public String title;
		public String address;
		public String provider;
		public String venue_id;
		public String phone_number;
		public String first_name;
		public String last_name;
		public String url;
		public String q;
	}

	public static class TL_inputMediaContact extends InputMedia {
	}

	public static class TL_inputMediaUploadedThumbDocument extends InputMedia {
	}

	public static class TL_inputMediaDocument extends InputMedia {
		public InputDocument id;
	}

	public static class TL_inputMediaGifExternal extends InputMedia {
	}

	public static class TL_inputMediaGeoPoint extends InputMedia {
	}

	public static class TL_inputMediaUploadedPhoto extends InputMedia {
	}

	public static class TL_inputMediaVenue extends InputMedia {
	}

	public static class TL_inputMediaUploadedDocument extends InputMedia {
	}

	public static class TL_inputMediaPhoto extends InputMedia {
		public InputPhoto id;
	}

	public static class InputStickerSet extends TLObject {
		public long id;
		public String short_name;
	}

	public static class TL_inputStickerSetEmpty extends InputStickerSet {
	}

	public static class InputEncryptedFile extends TLObject {
		public long id;
		public int parts;
	}

	public static class UserProfilePhoto extends TLObject {
		public FileLocation photo_small;
		public FileLocation photo_big;
	}

	public static class Photo extends TLObject {
		public long id;
		public long access_hash;
		public int user_id;
		public int date;
		public String caption;
		public ArrayList<PhotoSize> sizes = new ArrayList<>();
	}

	public static class TL_photo extends Photo {
	}

	public static class TL_photoEmpty extends Photo {
	}

	public static class Chat extends TLObject {
		public int flags;
		public boolean creator;
		public int id;
		public String title;
		public final ChatPhoto photo = null;
		public int date;
		public int version;
		public boolean editor;
		public String username;
		public boolean signatures;
		public boolean min;
		public String address;
	}

	public static class StickerSet extends TLObject {
		public long id;
		public String title;
		public int flags;
		public boolean installed;
		public boolean disabled;
		public boolean official;
		public int count;
		public int hash;
	}

	public static class TL_messageFwdHeader extends TLObject {
		public String m_name;
	}

	public static class FileLocation extends TLObject {
		public int dc_id;
		public long volume_id;
		public int local_id;
		public String mr_path; // if set, this superseeds the calculation of the path from dc_id/volume_id/local_id
		public byte[] key;
        public byte[] iv;
	}

	public static class TL_fileLocation extends FileLocation {
	}

	public static class TL_fileLocationUnavailable extends FileLocation {
	}

	public static class PhotoSize extends TLObject {
		public String type; // s, m, x, y, w for small, medium, ... widest
		public FileLocation location;
		public int w;
		public int h;
		public int size;
		public byte[] bytes;
	}

	public static class TL_photoSizeEmpty extends PhotoSize { // used as an indicator via instanceof
	}

	public static class TL_photoSize extends PhotoSize {
	}


	public static class TL_photoCachedSize extends PhotoSize {
	}

	public static class InputFile extends TLObject {
		public long id;
		public int parts;
		public String name;
	}

	public static class WallPaper extends TLObject {
		public int id;
		public String title;
		public ArrayList<PhotoSize> sizes = new ArrayList<>();
		public int color;
		public int bg_color;
	}

	public static class TL_wallPaper extends WallPaper {
	}

	public static class TL_wallPaperSolid extends WallPaper {
	}

	public static class InputPhoto extends TLObject {
		public long id;
		public long access_hash;
	}

	public static class TL_inputPhoto extends InputPhoto {
	}

	public static class InputPeer extends TLObject {
		public int user_id;
		public int chat_id;
		public int channel_id;
	}

	public static class TL_inputPeerUser extends InputPeer {
	}

	public static class MessageMedia extends TLObject {
		public byte[] bytes;
		public Photo photo;
		public GeoPoint geo;
		public String title;
		public String address;
		public String provider;
		public String venue_id;
		public Document document;
		public String caption;
		public String phone_number;
		public String first_name;
		public String last_name;
		public int user_id;
		public WebPage webpage;
	}

    public static class Message extends TLObject {
        public int id;
        public int from_id;
        public Peer to_id;
        public int date;
        public int reply_to_msg_id;
		public long reply_to_random_id;
        public String message;
        public MessageMedia media;
        public int flags;
		public boolean media_unread;
		public boolean out;
		public boolean unread;
		public int views;
		public boolean silent;
		public boolean post;
		public TL_messageFwdHeader fwd_from;
        public int send_state = 0; //custom
        public String attachPath = ""; //custom
		public HashMap<String, String> params; //custom
        public long random_id; //custom
        public long dialog_id; //custom
        public int layer; //custom
        public TLRPC.Message replyMessage; //custom
		public boolean created_by_mr;
    }

	public static class TL_messageEmpty extends Message {
	}

	public static class TL_message extends Message {
	}

	public static class TL_dialog extends TLObject {
		final public int flags = 0;
		public Peer peer;
		public int top_message;
		public int unread_count;
		public PeerNotifySettings notify_settings;
		public int pts;
		public DraftMessage draft;
		public int last_message_date; //custom
		public long id; //custom
	}

    public static class Vector extends TLObject {
        public ArrayList<Object> objects = new ArrayList<>();
    }
}
